@@ -81,15 +81,6 @@ public abstract class AbstractStarTable implements StarTable {
         parameters_ = parameters;
     }
 
-    public void setParameter( DescribedValue dval ) {
-        DescribedValue old = getParameterByName( dval.getInfo().getName() );
-        List<DescribedValue> parameters = getParameters();
-        if ( old != null ) {
-            parameters.remove( old );
-        }
-        parameters.add( dval );
-    }
-
     public String getName() {
         return name_;
     }
@@ -116,15 +107,6 @@ public abstract class AbstractStarTable implements StarTable {
         url_ = url;
     }
 
-    public DescribedValue getParameterByName( String parname ) {
-        for ( DescribedValue dval : getParameters() ) {
-            if ( parname.equals( dval.getInfo().getName() ) ) {
-                return dval;
-            }
-        }
-        return null;
-    }
-
     /**
      * Convenience method to get an <tt>int</tt> value from a <tt>long</tt>.
      * Invokes {@link Tables#checkedLongToInt}.
@@ -141,6 +123,23 @@ public abstract class AbstractStarTable implements StarTable {
         return false;
     }
 
+    public RowAccess getRowAccess() throws IOException {
+        throw new UnsupportedOperationException( "No random access available" );
+    }
+
+    /**
+     * Returns a default splittable which relies on table random access
+     * if available, or otherwise provides only sequential access (no splits).
+     *
+     * <p>It is often possible to provide a better implementation than this.
+     *
+     * @return  {@link Tables#getDefaultRowSplittable
+     *                 Tables.getDefaultRowSplittable(this)}
+     */
+    public RowSplittable getRowSplittable() throws IOException {
+        return Tables.getDefaultRowSplittable( this );
+    }
+
     /**
      * The <tt>AbstractStarTable</tt> implementation of this method throws an
      * <tt>UnsupportedOperationException</tt>, since unless otherwise
@@ -163,6 +162,13 @@ public abstract class AbstractStarTable implements StarTable {
         return row;
     }
 
+    /**
+     * The <tt>AbstractStarTable</tt> implementation of this method
+     * does nothing.
+     */
+    public void close() throws IOException {
+    }
+
     abstract public ColumnInfo getColumnInfo( int icol );
     abstract public int getColumnCount();
     abstract public long getRowCount();
