@@ -1,9 +1,14 @@
 package uk.ac.starlink.table;
 
 import java.lang.reflect.Array;
+import java.math.BigDecimal;
+import java.math.BigInteger;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
+import java.util.function.Function;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -16,16 +21,19 @@ import java.util.regex.Pattern;
  */
 public class DefaultValueInfo implements ValueInfo {
 
-    private String name;
-    private String unitString = null;
-    private String ucd = null;
-    private String utype = null;
-    private String description = "";
-    private Class<?> contentClass = Object.class;
-    private DomainMapper[] domainMappers = new DomainMapper[ 0 ];
-    private boolean isNullable = true;
-    private int[] shape = new int[] { -1 };
-    private int elementSize = -1;
+    private String name_;
+    private String unitString_;
+    private String ucd_;
+    private String utype_;
+    private String xtype_;
+    private String description_;
+    private Class<?> contentClass_;
+    private DomainMapper[] domainMappers_;
+    private boolean isNullable_;
+    private int[] shape_;
+    private int elementSize_;
+    private List<DescribedValue> auxData_;
+    private Function<String,?> fromString_;
 
     private static Pattern trailDigits = Pattern.compile( "\\.([0-9]+)$" );
     private static Pattern trailSpaces = Pattern.compile( "( +)$" );
@@ -35,6 +43,7 @@ public class DefaultValueInfo implements ValueInfo {
      * without a name.
      */
     public DefaultValueInfo() {
+        this( (String) null );
     }
 
     /**
@@ -44,7 +53,7 @@ public class DefaultValueInfo implements ValueInfo {
      * @param  name  the name applying to described values
      */
     public DefaultValueInfo( String name ) {
-        this.name = name;
+        this( name, Object.class );
     }
 
     /**
@@ -56,8 +65,7 @@ public class DefaultValueInfo implements ValueInfo {
      *         instances
      */
     public DefaultValueInfo( String name, Class<?> contentClass ) {
-        this( name );
-        setContentClass( contentClass );
+        this( name, contentClass, "" );
     }
 
     /**
@@ -71,9 +79,14 @@ public class DefaultValueInfo implements ValueInfo {
      */
     public DefaultValueInfo( String name, Class<?> contentClass,
                              String description ) {
-        this( name );
-        setContentClass( contentClass );
-        setDescription( description );
+        name_ = name;
+        description_ = description;
+        domainMappers_ = new DomainMapper[ 0 ];
+        isNullable_ = true;
+        shape_ = new int[] { -1 };
+        elementSize_ = -1;
+        auxData_ = new ArrayList<DescribedValue>();
+        configureContentClass( contentClass );
     }
 
     /**
@@ -84,16 +97,18 @@ public class DefaultValueInfo implements ValueInfo {
      * @param   base  the object to copy
      */
     public DefaultValueInfo( ValueInfo base ) {
-        this( base.getName() );
-        setUnitString( base.getUnitString() );
-        setUCD( base.getUCD() );
-        setUtype( base.getUtype() );
-        setDescription( base.getDescription() );
-        setContentClass( base.getContentClass() );
-        setShape( base.getShape() );
-        setElementSize( base.getElementSize() );
-        setNullable( base.isNullable() );
-        setDomainMappers( base.getDomainMappers() );
+        this( base.getName(), base.getContentClass(), base.getDescription() );
+        unitString_ = base.getUnitString();
+        ucd_ = base.getUCD();
+        utype_ = base.getUtype();
+        xtype_ = base.getXtype();
+        shape_ = base.getShape() == null ? null : base.getShape().clone();
+        elementSize_ = base.getElementSize();
+        isNullable_ = base.isNullable();
+        domainMappers_ = base.getDomainMappers() == null
+                       ? null
+                       : base.getDomainMappers().clone();
+        auxData_ = new ArrayList<DescribedValue>( base.getAuxData() );
     }
 
     /**
@@ -102,11 +117,11 @@ public class DefaultValueInfo implements ValueInfo {
      * @param   name  the name
      */
     public void setName( String name ) {
-        this.name = name;
+        name_ = name;
     }
 
     public String getName() {
-        return name;
+        return name_;
     }
 
     /**
@@ -117,11 +132,11 @@ public class DefaultValueInfo implements ValueInfo {
      *         units are unknown
      */
     public void setUnitString( String unitString ) {
-        this.unitString = unitString;
+        unitString_ = unitString;
     }
 
     public String getUnitString() {
-        return unitString;
+        return unitString_;
     }
 
     /**
@@ -131,11 +146,11 @@ public class DefaultValueInfo implements ValueInfo {
      * @param  ucd  the UCD, or <tt>null</tt> if none is known
      */
     public void setUCD( String ucd ) {
-        this.ucd = ucd;
+        ucd_ = ucd;
     }
 
     public String getUCD() {
-        return ucd;
+        return ucd_;
     }
 
     /**
@@ -144,11 +159,24 @@ public class DefaultValueInfo implements ValueInfo {
      * @param  utype  the Utype, or <code>null</code> if none is known
      */
     public void setUtype( String utype ) {
-        this.utype = utype;
+        utype_ = utype;
     }
 
     public String getUtype() {
-        return utype;
+        return utype_;
+    }
+
+    /**
+     * Sets the Xtype string applying to values described by this object.
+     *
+     * @param  xtype  the Xtype, or <code>null</code> if none is known
+     */
+    public void setXtype( String xtype ) {
+        xtype_ = xtype;
+    }
+
+    public String getXtype() {
+        return xtype_;
     }
 
     /**
@@ -158,15 +186,15 @@ public class DefaultValueInfo implements ValueInfo {
      *         or the empty string "" if there is nothing to be said
      */
     public void setDescription( String description ) {
-        this.description = description;
+        description_ = description;
     }
 
     public String getDescription() {
-        return description;
+        return description_;
     }
 
     public Class<?> getContentClass() {
-        return contentClass;
+        return contentClass_;
     }
 
     /**
@@ -176,34 +204,48 @@ public class DefaultValueInfo implements ValueInfo {
      * @throws  IllegalArgumentException if <tt>contentClass</tt> is primitive
      */
     public void setContentClass( Class<?> contentClass ) {
+        configureContentClass( contentClass );
+    }
+
+    /**
+     * Does the work for setting the content class of this object.
+     * This is a final method that is safe to call from the constructor.
+     *
+     * @param  contentClass  the class of items in this column
+     * @throws  IllegalArgumentException if <tt>contentClass</tt> is primitive
+     */
+    private final void configureContentClass( Class<?> contentClass ) {
         if ( contentClass.isPrimitive() ) {
             throw new IllegalArgumentException( 
                 "Primitive content class " + contentClass + " not permitted" );
         }
 
         /* Set the class. */
-        this.contentClass = contentClass;
+        contentClass_ = contentClass;
 
         /* If the content class is not an array type, set the shape to null. */
         if ( ! isArray() ) {
-            setShape( null );
+            shape_ = null;
         }
 
         /* If it is an array, ensure that the shape array is consistent
          * with this. */
         else {
-            if ( shape == null ) {
-                setShape( new int[] { -1 } );
+            if ( shape_ == null ) {
+                shape_ = new int[] { -1 };
             }
         }
+
+        /* Prepare for conversion from string values. */
+        fromString_ = createUnformatter( contentClass );
     }
 
     public boolean isArray() {
-        return contentClass.isArray();
+        return contentClass_.isArray();
     }
 
     public int[] getShape() {
-        return shape == null ? null : shape.clone();
+        return shape_ == null ? null : shape_.clone();
     }
 
     /**
@@ -221,11 +263,11 @@ public class DefaultValueInfo implements ValueInfo {
                 }
             }
         }
-        this.shape = shape;
+        shape_ = shape;
     }
 
     public int getElementSize() {
-        return elementSize;
+        return elementSize_;
     }
 
     /**
@@ -234,11 +276,11 @@ public class DefaultValueInfo implements ValueInfo {
      * @param  size  the element size
      */
     public void setElementSize( int size ) {
-        this.elementSize = size;
+        elementSize_ = size;
     }
 
     public boolean isNullable() {
-        return isNullable;
+        return isNullable_;
     }
 
     /**
@@ -250,11 +292,11 @@ public class DefaultValueInfo implements ValueInfo {
      *         guaranteed non-<tt>null</tt>
      */
     public void setNullable( boolean isNullable ) {
-        this.isNullable = isNullable;
+        isNullable_ = isNullable;
     }
 
     public DomainMapper[] getDomainMappers() {
-        return domainMappers;
+        return domainMappers_;
     }
 
     /**
@@ -263,7 +305,28 @@ public class DefaultValueInfo implements ValueInfo {
      * @param  domainMappers  new domain mapper array
      */
     public void setDomainMappers( DomainMapper[] domainMappers ) {
-        this.domainMappers = domainMappers;
+        domainMappers_ = domainMappers;
+    }
+
+    /**
+     * Returns a list of auxiliary metadata objects
+     * pertaining to this column.
+     * The returned value may, if mutable, be modified to change
+     * the aux data of this object.
+     *
+     * @return   a List of <tt>DescribedValue</tt> items
+     */
+    public List<DescribedValue> getAuxData() {
+        return auxData_;
+    }
+
+    /**
+     * Sets the list of auxiliary metadata items for this column.
+     *
+     * @param   auxData  a list of <tt>DescribedValue</tt> objects
+     */
+    public void setAuxData( List<DescribedValue> auxData ) {
+        auxData_ = auxData;
     }
 
     /**
@@ -396,7 +459,7 @@ public class DefaultValueInfo implements ValueInfo {
         /* Otherwise, it's an array, output it in a multidimensional array
          * if necessary. */
         else {
-            int[] ashape = getActualShape( shape, Array.getLength( value ) );
+            int[] ashape = getActualShape( shape_, Array.getLength( value ) );
             appendElements( buf, value, 0, ashape, maxLength );
         }
     
@@ -590,70 +653,11 @@ public class DefaultValueInfo implements ValueInfo {
     }
 
     public Object unformatString( String rep ) {
-
-        /*
-         * This is a not-very-OO stopgap solution to this problem.
-         * the proper solution requires some significant rejigging of
-         * the class hierarchy in the tables package. 
-         * I am writing demos that need to be working in a few days
-         * and don't have time for it now.  mbt.
-         */
-        if ( rep == null || rep.length() == 0 ) {
-            return null;
-        }
-        Class<?> clazz = getContentClass();
-        if ( clazz == Boolean.class ) {
-            return Boolean.valueOf( rep );
-        }
-        else if ( clazz == Character.class ) {
-            if ( rep.length() == 1 ) {
-                return new Character( rep.charAt( 0 ) );
-            }
-            else if ( rep.trim().length() == 1 ) {
-                return new Character( rep.trim().charAt( 0 ) );
-            }
-            else {
-                throw new IllegalArgumentException();
-            }
-        }
-        else if ( clazz == Byte.class ) {
-            return Byte.valueOf( rep );
-        }
-        else if ( clazz == Short.class ) {
-            return Short.valueOf( rep );
-        }
-        else if ( clazz == Integer.class ) {
-            return Integer.valueOf( rep );
-        }
-        else if ( clazz == Long.class ) {
-            return Long.valueOf( rep );
-        }
-        else if ( clazz == Float.class ) {
-            if ( rep.trim().length() == 0 ) {
-                return new Float( Float.NaN );
-            }
-            else {
-                return Float.valueOf( rep );
-            }
-        }
-        else if ( clazz == Double.class || clazz == Number.class ) {
-            if ( rep.trim().length() == 0 ) {
-                return new Double( Double.NaN );
-            }
-            else {
-                return Double.valueOf( rep );
-            }
-        }
-        else if ( clazz == String.class ) {
-            return rep;
-        }
-        else {
-            throw new IllegalArgumentException(
-                "No unformatter available for " + clazz.getName() );
-        }
+        return rep == null || rep.length() == 0
+             ? null
+             : fromString_.apply( rep );
     }
 
-        
     /**
      * Returns a string representation of this object.
      * The result indicates the object's name, class and shape. 
@@ -700,4 +704,67 @@ public class DefaultValueInfo implements ValueInfo {
         return ashape;
     }
 
+    /**
+     * Returns a function that can convert from a string to a given class.
+     * The output function has the same output type as the parameterised
+     * type of the input class, but that is not not enforced by the
+     * signature becuase it's a pain to implement like that.
+     *
+     * @param  clazz   required function output class
+     * @return  function that converts a string to the required class
+     */
+    private static Function<String,?> createUnformatter( Class<?> clazz ) {
+        if ( Boolean.class.equals( clazz ) ) {
+            return Boolean::valueOf;
+        }
+        else if ( Character.class.equals( clazz ) ) {
+            return txt -> {
+                if ( txt.length() == 1 ) {
+                    return Character.valueOf( txt.charAt( 0 ) );
+                }
+                else if ( txt.trim().length() == 1 ) {
+                    return Character.valueOf( txt.trim().charAt( 0 ) );
+                }
+                else {
+                    throw new IllegalArgumentException( "Not char" );
+                }
+            };
+        }
+        else if ( Byte.class.equals( clazz ) ) {
+            return Byte::valueOf;
+        }
+        else if ( Short.class.equals( clazz ) ) {
+            return Short::valueOf;
+        }
+        else if ( Integer.class.equals( clazz ) ) {
+            return Integer::valueOf;
+        }
+        else if ( Long.class.equals( clazz ) ) {
+            return Long::valueOf;
+        }
+        else if ( Float.class.equals( clazz ) ) {
+            return Float::valueOf;
+        }
+        else if ( Double.class.equals( clazz ) ) {
+            return Double::valueOf;
+        }
+        else if ( BigInteger.class.equals( clazz ) ) {
+            return BigInteger::new;
+        }
+        else if ( BigDecimal.class.equals( clazz ) ) {
+            return BigDecimal::new;
+        }
+        else if ( String.class.equals( clazz ) ) {
+            return Function.identity();
+        }
+        else if ( Number.class.isAssignableFrom( clazz ) ) {
+            return Double::valueOf;
+        }
+        else {
+            return txt -> {
+                throw new UnsupportedOperationException( "No unformatter"
+                                                       + " available" );
+            };
+        }
+    }
 }
