@@ -23,7 +23,7 @@ public class OnceRowPipe implements RowPipe, RowSequence {
     private Object[] seqRow_;
     private boolean seqClosed_;
     private boolean seqEnded_;
-    private IOException error_;
+    private volatile IOException error_;
 
     private static final Object[] END_ROWS = new Object[ 0 ];
 
@@ -46,6 +46,7 @@ public class OnceRowPipe implements RowPipe, RowSequence {
     }
 
     public synchronized void setError( IOException error ) {
+        rowQueue_.clear();
         if ( error_ == null ) {
             error_ = error;
             notifyAll();
@@ -58,6 +59,12 @@ public class OnceRowPipe implements RowPipe, RowSequence {
             public boolean isRandom() {
                 return false;
             }
+            public RowAccess getRowAccess() {
+                throw new UnsupportedOperationException();
+            }
+            public RowSplittable getRowSplittable() throws IOException {
+                return Tables.getDefaultRowSplittable( this );
+            }
             public RowSequence getRowSequence() throws IOException {
                 synchronized ( OnceRowPipe.this ) {
                     if ( rseq_ == null ) {
@@ -97,6 +104,7 @@ public class OnceRowPipe implements RowPipe, RowSequence {
         if ( seqClosed_ ) {
             throw new IOException( "Stream closed at reading end" );
         }
+        checkError();
         try {
             while ( rowQueue_.size() > queueSize_ ) {
                 wait();
