@@ -1,13 +1,20 @@
 package uk.ac.starlink.table;
 
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.lang.reflect.Array;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.BitSet;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
+import java.util.function.LongSupplier;
+import uk.ac.starlink.table.StarTableOutput;
+import uk.ac.starlink.table.StarTableWriter;
+import uk.ac.starlink.table.formats.TextTableWriter;
 import uk.ac.starlink.table.jdbc.JDBCStarTable;
 
 /**
@@ -56,10 +63,6 @@ public class Tables {
         new DefaultValueInfo( "UBYTE_FLAG", Boolean.class,
                               "If true, data represents unsigned byte values" );
 
-    /** ValueInfo for Extended Type (VOTable <code>xtype</code>) attribute. */
-    public static final ValueInfo XTYPE_INFO = new DefaultValueInfo(
-        "xtype", String.class, "Extended type information" );
-
     /**
      * ValueInfo representing Right Ascension. 
      * The units are radians and it is non-nullable.
@@ -168,6 +171,22 @@ public class Tables {
         sink.endRows();
     }
 
+    /**
+     * Returns a RowSplittable object with generic characteristics
+     * for a given table.
+     * For a random-access table the splitting will be based on
+     * {@link RowAccess} objects, and for a non-random table
+     * it will not be capable of splits.
+     *
+     * @param  table  table
+     * @return  splittable for potentially parallel iteration over rows
+     */
+    public static RowSplittable getDefaultRowSplittable( StarTable table )
+            throws IOException {
+        return table.isRandom() ? new RandomRowSplittable( table )
+                                : new SequentialRowSplittable( table );
+    }
+
     /**
      * Diagnostic method which tests the invariants of a StarTable.
      * This method returns no value, and throws an exception if a table
@@ -196,6 +215,9 @@ public class Tables {
         /* Check a random-access table knows how many rows it has. */
         if ( isRandom ) {
             assertTrue( nrow >= 0 );
+            RowAccess racc = table.getRowAccess();
+            assertTrue( racc != null );
+            racc.close();
         }
 
         /* Check the shape arrays look OK. */
@@ -236,8 +258,12 @@ public class Tables {
 
         /* Read all cells. */
         long lrow = 0L;
+        RowAccess racc = isRandom ? table.getRowAccess() : null;
+        RowSplittable rsplit = table.getRowSplittable();
         while ( rseq.next() ) {
+            assertTrue( rsplit.next() );
             Object[] row = rseq.getRow();
+            Object[] rowSplit = rsplit.getRow(); 
             for ( int icol = 0; icol < ncol; icol++ ) {
 
                 /* Check that elements got in different ways look the same. */
@@ -245,26 +271,41 @@ public class Tables {
                 Object val1 = cell;
                 Object val2 = rseq.getCell( icol );
                 Object val3 = null;
+                Object val4 = null;
+                Object val5 = rowSplit[ icol ];
+                Object val6 = rsplit.getCell( icol );
                 if ( isRandom ) {
                     val3 = table.getCell( lrow, icol );
+                    racc.setRowIndex( lrow );
+                    val4 = racc.getCell( icol );
                 }
                 boolean isNull = cell == null;
                 if ( isNull ) {
                     assertTrue( colinfos[ icol ].isNullable() );
                     assertTrue( val2 == null );
+                    assertTrue( val5 == null );
+                    assertTrue( val6 == null );
                     if ( isRandom ) {
-                        assertTrue( val3 == null);
+                        assertTrue( val3 == null );
+                        assertTrue( val4 == null );
                     }
                 }
                 else {
-                    String s1 = colinfos[ icol ]
-                               .formatValue( val1, formatChars );
-                    assertTrue( s1.equals( colinfos[ icol ]
+                    ColumnInfo cinfo = colinfos[ icol ];
+                    String s1 = cinfo.formatValue( val1, formatChars );
+                    assertTrue( s1.equals( cinfo
                                           .formatValue( val2, formatChars ) ) );
+                    assertTrue( s1.equals( cinfo
+                                          .formatValue( val5, formatChars ) ) );
+                    assertTrue( s1.equals( cinfo
+                                          .formatValue( val6, formatChars ) ) );
                     if ( isRandom ) {
                         assertTrue( s1.equals( colinfos[ icol ]
                                               .formatValue( val3,
                                                             formatChars ) ) );
+                        assertTrue( s1.equals( colinfos[ icol ]
+                                              .formatValue( val4,
+                                                            formatChars ) ) );
                     }
                 }
 
@@ -292,6 +333,42 @@ public class Tables {
             lrow++;
         }
         rseq.close();
+        assertTrue( ! rsplit.next() );
+        rsplit.close();
+
+        /* Check splittables split correctly. */
+        RowSplittable split1 = table.getRowSplittable();
+        RowSplittable split2 = split1.split();
+        LongSupplier rowIndex1 = split1.rowIndex();
+        boolean hasRowIndex = rowIndex1 != null;
+        if ( split2 != null ) {
+            LongSupplier rowIndex2 = split2.rowIndex();
+            assertTrue( ( rowIndex2 != null ) == hasRowIndex );
+            BitSet bits = hasRowIndex && nrow < Integer.MAX_VALUE
+                        ? new BitSet( nrow >= 0 ? (int) nrow : 1024 )
+                        : null;
+            int nr = 0;
+            while ( split2.next() ) {
+                nr++;
+                if ( bits != null ) {
+                    bits.set( (int) rowIndex2.getAsLong() );
+                }
+            }
+            while ( split1.next() ) {
+                nr++;
+                if ( bits != null ) {
+                    bits.set( (int) rowIndex1.getAsLong() );
+                }
+            }
+            split2.close();
+            if ( nrow >= 0 ) {
+                assertTrue( nrow == nr );
+            }
+            if ( bits != null ) {
+                assertTrue( nr == bits.nextClearBit( 0 ) );
+            }
+        }
+        split1.close();
 
         /* Check that the claimed number of rows is correct. */
         if ( nrow >= 0 ) {
@@ -390,6 +467,42 @@ public class Tables {
         return new RowPermutedStarTable( table, rowMap );
     }
 
+    /**
+     * Returns the contents of a table as a string.
+     * Only intended for small tables, for instance during debugging.
+     *
+     * @param  table   input table
+     * @param  ofmt   output format specifier, or null for plain text output
+     * @return  stringified table
+     */
+    public static String tableToString( StarTable table, String ofmt ) {
+        final StarTableWriter handler;
+        if ( ofmt != null ) {
+            try {
+                handler = new StarTableOutput().getHandler( ofmt );
+            }
+            catch ( TableFormatException e ) {
+                throw new IllegalArgumentException( "No such format \"" + ofmt
+                                                  + "\"", e );
+            }
+        }
+        else {
+            TextTableWriter textHandler = new TextTableWriter();
+            textHandler.setWriteParameters( false );
+            textHandler.setMaxWidth( 40 );
+            handler = textHandler;
+        }
+        try {
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            handler.writeStarTable( table, out );
+            out.close();
+            return new String( out.toByteArray(), "utf-8" );
+        }
+        catch ( IOException e ) {
+            return "Formatting error: " + e;
+        }
+    }
+
     /**
      * Convenience method to get an <tt>int</tt> value from a <tt>long</tt>.
      * If the supplied long integer <tt>lval</tt> is out of the range
@@ -479,22 +592,6 @@ public class Tables {
         return info.getUtype();
     }
 
-    /**
-     * Returns the extended type value associated with a given metadata item.
-     * The XType corresponds to the <code>xtype</code> attribute of
-     * the VOTable format.  Other table formats may or may not be able
-     * to represent it.
-     *
-     * @param  info  metadata item
-     * @return   xtype string, or null if none is available
-     */
-    public static String getXtype( ValueInfo info ) {
-        return info instanceof ColumnInfo
-             ? (String) ((ColumnInfo) info)
-                       .getAuxDatumValue( XTYPE_INFO, String.class )
-             : null;
-    }
-
     /**
      * Tries to set the Utype for a given metadata item.
      *
@@ -510,6 +607,103 @@ public class Tables {
         }
     }
 
+    /**
+     * Returns the extended type value associated with a given metadata item.
+     * The XType corresponds to the <code>xtype</code> attribute of
+     * the VOTable format.  Other table formats may or may not be able
+     * to represent it.
+     *
+     * @deprecated  use {@link ValueInfo#getXtype} instead
+     * @param  info  metadata item
+     * @return   xtype string, or null if none is available
+     */
+    @Deprecated
+    public static String getXtype( ValueInfo info ) {
+        return info.getXtype();
+    }
+
+    /**
+     * Tries to set the Xtype for a given metadata item.
+     *
+     * @deprecated  use {@link DefaultValueInfo#setXtype} instead
+     * @param  info  metadata item
+     * @param  xtype  new xtype value
+     * @see   #getXtype
+     */
+    @Deprecated
+    public static void setXtype( ValueInfo info, String xtype ) {
+        if ( info instanceof DefaultValueInfo ) {
+            ((DefaultValueInfo) info).setXtype( xtype );
+        }
+    }
+
+    /**
+     * Utility method to update a list of DescribedValues with a new entry.
+     * If an item with the same name as the new entry already exists,
+     * it is removed.
+     *
+     * @param  dvals  list to modify
+     * @param  dval  new entry to add
+     */
+    public static void setDescribedValue( Collection<DescribedValue> dvals,
+                                          DescribedValue dval ) {
+        DescribedValue old =
+            getDescribedValueByName( dvals, dval.getInfo().getName() );
+        if ( old != null ) {
+            dvals.remove( old );
+        }
+        dvals.add( dval );
+    }
+
+    /**
+     * Utility method to locate an element in a list of DescribedValue
+     * given the name of its ValueInfo member.
+     *
+     * @param   dvals  list to query
+     * @param   name   required value of name
+     * @return  element of <code>dvals</code> for which
+     *                  <code>getInfo().getName()</code>
+     *                  matches <code>name</code>
+     */
+    public static DescribedValue
+            getDescribedValueByName( Collection<DescribedValue> dvals,
+                                     String name ) {
+        for ( DescribedValue dval : dvals ) {
+            if ( name.equals( dval.getInfo().getName() ) ) {
+                return dval;
+            }
+        }
+        return null;
+    }
+
+    /**
+     * Utility method to return an auxiliary metadata item from a ValueInfo.
+     *
+     * @param  info   metadata item
+     * @param  auxKey   info identifying aux metadata entry in <code>info</code>
+     * @param  auxClazz   required result type
+     * @return   typed aux metadata item requested,
+     *           or null if it doesn't exist or has the wrong type
+     */
+    public static <T> T getAuxDatumValue( ValueInfo info, ValueInfo auxKey,
+                                          Class<T> auxClazz ) {
+        return getTypedValue( info.getAuxDatumByName( auxKey.getName() ),
+                              auxClazz );
+    }
+
+    /**
+     * Utility method to get a typed value from a, possibly null,
+     * described value.
+     *
+     * @param  dval   described value, or null
+     * @param  clazz   required return type
+     * @return   typed value of <code>dval</code>, or null if dval is null,
+     *           or if dval's value has the wrong type
+     */
+    public static <T> T getTypedValue( DescribedValue dval, Class<T> clazz ) {
+        return dval == null ? null : dval.getTypedValue( clazz );
+    }
+
     /**
      * Returns the value from a list of {@link DescribedValue} objects
      * which corresponds to a given info key.
