@@ -13,7 +13,6 @@ import uk.ac.starlink.table.StarTableOutput;
  * @author   Mark Taylor (Starlink)
  * @since    25 Feb 2005
  */
-@SuppressWarnings("rawtypes")
 public interface TableSaveDialog {
 
     /**
@@ -63,5 +62,6 @@ public interface TableSaveDialog {
      * @return   true iff the save completed successfully
      */
     boolean showSaveDialog( Component parent, StarTableOutput sto,
-                            ComboBoxModel formatModel, StarTable[] tables );
+                            ComboBoxModel<String> formatModel,
+                            StarTable[] tables );
 }
