@@ -7,7 +7,10 @@ import java.nio.ByteBuffer;
 import java.util.logging.Logger;
 import uk.ac.starlink.table.ByteStore;
 import uk.ac.starlink.table.ColumnInfo;
+import uk.ac.starlink.table.RandomRowSplittable;
+import uk.ac.starlink.table.RowAccess;
 import uk.ac.starlink.table.RowSequence;
+import uk.ac.starlink.table.RowSplittable;
 import uk.ac.starlink.table.RowStore;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.TableFormatException;
@@ -138,7 +141,12 @@ public class ByteStoreRowStore implements RowStore {
         }
         logger_.config( nrow_ + " rows stored in " + nbyte + " bytes" );
         storedTable_ = new ByteStoreStarTable( template_, nrow_, codecs_,
-                                               offsets_, bbufs );
+                                               offsets_, bbufs ) {
+            @Override
+            public void close() throws IOException {
+                byteStore_.close();
+            }
+        };
     }
 
     public StarTable getStarTable() {
@@ -260,6 +268,34 @@ public class ByteStoreRowStore implements RowStore {
             };
         }
 
+        public RowSplittable getRowSplittable() throws IOException {
+            return new RandomRowSplittable( this );
+        }
+
+        public RowAccess getRowAccess() throws IOException {
+            final ByteStoreAccess access = createAccess();
+            final Object[] row = new Object[ ncol_ ];
+            return new RowAccess() {
+                long irow_ = -1;
+                public void setRowIndex( long irow ) {
+                    irow_ = irow;
+                }
+                public Object getCell( int icol ) throws IOException {
+                    access.seek( offsets_.getCellOffset( irow_, icol ) );
+                    return codecs_[ icol ].decodeObject( access );
+                }
+                public Object[] getRow() throws IOException {
+                    access.seek( offsets_.getRowOffset( irow_ ) );
+                    for ( int icol = 0; icol < ncol_; icol++ ) {
+                        row[ icol ] = codecs_[ icol ].decodeObject( access );
+                    }
+                    return row;
+                }
+                public void close() {
+                }
+            };
+        }
+
         /**
          * Creates a ByteStoreAccess that can access the data
          * for this row store.
