@@ -10,10 +10,11 @@ import uk.ac.starlink.table.ValueInfo;
  * Serializes and deserializes objects to/from a data stream.
  * A given instance of this class is only able to de/serialize 
  * an object of one class.  Obtain an instance from the
- * static {@link #getCodec} method.
+ * static {@link #getCodec} method, or use one of the static members.
+ * The static members are all safe for concurrent use from multiple threads.
  * 
  * <p>This is (supposed to be) considerably more lightweight than 
- * all the {@link java.lang.Serializable} business - for one thing
+ * all the {@link java.io.Serializable} business - for one thing
  * there's no way to tell from the stream what kind of item has been
  * serialized, you have to make sure the right Codec instance is on
  * hand.  In general it deals with primitive wrapper objects and
@@ -23,12 +24,63 @@ import uk.ac.starlink.table.ValueInfo;
  * @author   Mark Taylor (Starlink)
  * @since    3 Aug 2004
  */
-abstract class Codec {
+public abstract class Codec {
 
     private int warnings_;
     private static Logger logger_ =
         Logger.getLogger( "uk.ac.starlink.table.storage" );
 
+    /** Codec for 8-bit byte, no null handling. */
+    public static final Codec BYTE = new ByteCodec();
+
+    /** Codec for 16-bit integer, no null handling. */
+    public static final Codec SHORT = new ShortCodec();
+
+    /** Codec for 32-bit integer, no null handling. */
+    public static final Codec INT = new IntCodec();
+
+    /** Codec for 64-bit integer, no null handling. */
+    public static final Codec LONG = new LongCodec();
+
+    /** Codec for 32-bit floating point, nulls treated like NaN. */
+    public static final Codec FLOAT = new FloatCodec();
+
+    /** Codec for 64-bit floating point, nulls treated like NaN. */
+    public static final Codec DOUBLE = new DoubleCodec();
+
+    /** Codec for 16-bit character, no null handling. */
+    public static final Codec CHAR = new CharCodec();
+
+    /** Codec for byte-serialized boolean values, null handled. */
+    public static final Codec BOOLEAN = new BooleanCodec();
+
+    /** Codec for variable-length array of 8-bit bytes. */
+    public static final Codec BYTE_ARRAY =
+        new VariableArrayCodec( new ByteCodec1() );
+
+    /** Codec for variable-length array of 16-bit integers. */
+    public static final Codec SHORT_ARRAY =
+        new VariableArrayCodec( new ShortCodec1() );
+
+    /** Codec for variable-length array of 32-bit integers. */
+    public static final Codec INT_ARRAY =
+        new VariableArrayCodec( new IntCodec1() );
+
+    /** Codec for variable-length array of 64-bit integers. */
+    public static final Codec LONG_ARRAY =
+        new VariableArrayCodec( new LongCodec1() );
+
+    /** Codec for variable-length array of 32-bit floats. */
+    public static final Codec FLOAT_ARRAY =
+        new VariableArrayCodec( new FloatCodec1() );
+
+    /** Codec for variable-length array of 64-bit doubles. */
+    public static final Codec DOUBLE_ARRAY =
+        new VariableArrayCodec( new DoubleCodec1() );
+
+    /** Codec for variable-length string. */
+    public static final Codec STRING = new VariableStringCodec();
+
     /**
      * Serializes an object to a stream.
      *
@@ -40,20 +92,65 @@ abstract class Codec {
             throws IOException;
 
     /**
-     * Deserializes an object from a stream.
+     * Deserializes an item from a stream.
      *
-     * @param  in  source stream, positioned at start of object
-     * @return  deserialized object
+     * @param  in  source stream, positioned at start of item
+     * @return  deserialized item
      */
-    abstract public Object decode( ByteStoreAccess in ) throws IOException;
+    abstract public Object decodeObject( ByteStoreAccess in )
+            throws IOException;
+
+    /**
+     * Deserialises an item from a stream, and presents it as an integer
+     * if possible.  If the stream does not contain integer-like items,
+     * the result may not be useful.
+     *
+     * @param  in  source stream, positioned at start of item
+     * @return  best-effort integer equivalent of deserialised item
+     */
+    abstract public int decodeInt( ByteStoreAccess in )
+            throws IOException;
+
+    /**
+     * Deserialises an item from a stream, and presents it as a long integer
+     * if possible.  If the stream does not contain integer-like items,
+     * the result may not be useful.
+     *
+     * @param  in  source stream, positioned at start of item
+     * @return  best-effort long equivalent of deserialised item
+     */
+    abstract public long decodeLong( ByteStoreAccess in )
+            throws IOException;
+
+    /**
+     * Deserialises an item from a stream, and presents it as a double
+     * if possible.  If the stream does not contain numeric items,
+     * the result may not be useful.
+     *
+     * @param  in  source stream, positioned at start of item
+     * @return  best-effort double equivalent of deserialised item
+     */
+    abstract public double decodeDouble( ByteStoreAccess in )
+            throws IOException;
+
+    /**
+     * Deserialises an item from a stream, and presents it as a boolean
+     * if possible.  If the stream does not contain boolean-like items,
+     * the result may not be useful.
+     *
+     * @param  in  source stream, positioned at start of item
+     * @return  best-effort boolean equivalent of deserialised item
+     */
+    abstract public boolean decodeBoolean( ByteStoreAccess in )
+            throws IOException;
 
     /**
      * Returns the number of bytes a call to <tt>encode</tt> will write.
      * If this value may vary, -1 is returned.
      *
-     * @param  size in bytes of serialized items, or -1
+     * @return  size in bytes of serialized items, or -1
      */
-    abstract int getItemSize();
+    abstract public int getItemSize();
 
     /**
      * Returns a codec suitable for serializing/deserializing the contents
@@ -67,28 +164,28 @@ abstract class Codec {
     public static Codec getCodec( ValueInfo info ) {
         Class<?> clazz = info.getContentClass();
         if ( clazz == Byte.class ) {
-            return new FlaggedCodec( new ByteCodec() );
+            return new FlaggedCodec( BYTE );
         }
         else if ( clazz == Short.class ) {
-            return new FlaggedCodec( new ShortCodec() );
+            return new FlaggedCodec( SHORT );
         }
         else if ( clazz == Integer.class ) {
-            return new FlaggedCodec( new IntCodec() );
+            return new FlaggedCodec( INT );
         }
         else if ( clazz == Long.class ) {
-            return new FlaggedCodec( new LongCodec() );
+            return new FlaggedCodec( LONG );
         }
         else if ( clazz == Character.class ) {
-            return new FlaggedCodec( new CharCodec() );
+            return new FlaggedCodec( CHAR );
         }
         else if ( clazz == Float.class ) {
-            return new FloatCodec();
+            return FLOAT;
         }
         else if ( clazz == Double.class ) {
-            return new DoubleCodec();
+            return DOUBLE;
         }
         else if ( clazz == Boolean.class ) {
-            return new BooleanCodec();
+            return BOOLEAN;
         }
         else if ( clazz == String.class ) {
             int esize = info.getElementSize();
@@ -162,12 +259,24 @@ abstract class Codec {
 
     private static class ByteCodec extends Codec {
         public int encode( Object value, DataOutput out ) throws IOException {
-            out.writeByte( ((Byte) value).byteValue() );
+            out.writeByte( ((Number) value).byteValue() );
             return 1;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Byte( in.readByte() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return in.readByte();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return in.readByte();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return in.readByte();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return in.readByte() != 0;
+        }
         public int getItemSize() {
             return 1;
         }
@@ -175,12 +284,24 @@ abstract class Codec {
 
     private static class ShortCodec extends Codec {
         public int encode( Object value, DataOutput out ) throws IOException {
-            out.writeShort( ((Short) value).shortValue() );
+            out.writeShort( ((Number) value).shortValue() );
             return 2;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Short( in.readShort() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return in.readShort();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return in.readShort();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return in.readShort();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return in.readShort() != 0;
+        }
         public int getItemSize() {
             return 2;
         }
@@ -188,12 +309,24 @@ abstract class Codec {
 
     private static class IntCodec extends Codec {
         public int encode( Object value, DataOutput out ) throws IOException {
-            out.writeInt( ((Integer) value).intValue() );
+            out.writeInt( ((Number) value).intValue() );
             return 4;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Integer( in.readInt() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return in.readInt();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return in.readInt();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return in.readInt();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return in.readInt() != 0;
+        }
         public int getItemSize() {
             return 4;
         }
@@ -201,12 +334,24 @@ abstract class Codec {
 
     private static class LongCodec extends Codec {
         public int encode( Object value, DataOutput out ) throws IOException {
-            out.writeLong( ((Long) value).longValue() );
+            out.writeLong( ((Number) value).longValue() );
             return 8;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Long( in.readLong() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return (int) in.readLong();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return in.readLong();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return in.readLong();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return in.readLong() != 0;
+        }
         public int getItemSize() {
             return 8;
         }
@@ -217,9 +362,21 @@ abstract class Codec {
             out.writeChar( ((Character) value).charValue() );
             return 2;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Character( in.readChar() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return (int) in.readChar();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return (long) in.readChar();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return (double) in.readChar();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return in.readChar() != '\0';
+        }
         public int getItemSize() {
             return 2;
         }
@@ -228,12 +385,24 @@ abstract class Codec {
     private static class FloatCodec extends Codec {
         public int encode( Object value, DataOutput out ) throws IOException {
             out.writeFloat( value == null ? Float.NaN
-                                          : ((Float) value).floatValue() );
+                                          : ((Number) value).floatValue() );
             return 4;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Float( in.readFloat() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return (int) in.readFloat();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return (long) in.readFloat();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return in.readFloat();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return ! ( in.readFloat() == 0 );
+        }
         public int getItemSize() {
             return 4;
         }
@@ -242,12 +411,24 @@ abstract class Codec {
     private static class DoubleCodec extends Codec {
         public int encode( Object value, DataOutput out ) throws IOException {
             out.writeDouble( value == null ? Double.NaN
-                                           : ((Double) value).doubleValue() );
+                                           : ((Number) value).doubleValue() );
             return 8;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             return new Double( in.readDouble() );
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return (int) in.readDouble();
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return (long) in.readDouble();
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            return in.readDouble();
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return ! ( in.readDouble() == 0 );
+        }
         public int getItemSize() {
             return 8;
         }
@@ -262,7 +443,7 @@ abstract class Codec {
             out.writeByte( brep );
             return 1;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Boolean decodeObject( ByteStoreAccess in ) throws IOException {
             switch ( in.readByte() ) {
                 case (byte) 'T':
                     return Boolean.TRUE;
@@ -274,6 +455,27 @@ abstract class Codec {
             this.warnCorrupt();
             return null;
         }
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            return decodeBoolean( in ) ? 1 : 0;
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            return decodeBoolean( in ) ? 1 : 0;
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            switch ( in.readByte() ) {
+                case (byte) 'T':
+                    return 1.0;
+                case (byte) 'F':
+                    return 0.0;
+                case (byte) ' ':
+                    return Double.NaN;
+            }
+            this.warnCorrupt();
+            return Double.NaN;
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            return in.readByte() == 'T';
+        }
         public int getItemSize() {
             return 1;
         }
@@ -308,17 +510,74 @@ abstract class Codec {
             return itemSize_;
         }
 
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             byte flag = in.readByte();
             switch ( flag ) {
                 case OK:
-                    return baseCodec_.decode( in );
+                    return baseCodec_.decodeObject( in );
                 case BAD: 
                     in.skip( itemSize_ - 1 );
                     return null;
+                default:
+                    this.warnCorrupt();
+                    return null;
+            }
+        }
+
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            byte flag = in.readByte();
+            switch ( flag ) {
+                case OK:
+                    return baseCodec_.decodeInt( in );
+                case BAD:
+                    in.skip( itemSize_ - 1 );
+                    return 0;
+                default:
+                    this.warnCorrupt();
+                    return 0;
+            }
+        }
+
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            byte flag = in.readByte();
+            switch ( flag ) {
+                case OK:
+                    return baseCodec_.decodeLong( in );
+                case BAD:
+                    in.skip( itemSize_ - 1 );
+                    return 0;
+                default:
+                    this.warnCorrupt();
+                    return 0;
+            }
+        }
+
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            byte flag = in.readByte();
+            switch ( flag ) {
+                case OK:
+                    return baseCodec_.decodeDouble( in );
+                case BAD:
+                    in.skip( itemSize_ - 1 );
+                    return Double.NaN;
+                default:
+                    this.warnCorrupt();
+                    return Double.NaN;
+            }
+        }
+
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            byte flag = in.readByte();
+            switch ( flag ) {
+                case OK:
+                    return baseCodec_.decodeBoolean( in );
+                case BAD:
+                    in.skip( itemSize_ - 1 );
+                    return false;
+                default:
+                    this.warnCorrupt();
+                    return false;
             }
-            this.warnCorrupt();
-            return null;
         }
 
         public int getItemSize() {
@@ -326,10 +585,33 @@ abstract class Codec {
         }
     }
 
+    /**
+     * Partial Codec implementation for codecs that are dealing with objects
+     * that have no reasonable primitive interpretation.
+     */
+    private static abstract class ObjectCodec extends Codec {
+        public int decodeInt( ByteStoreAccess in ) throws IOException {
+            decodeObject( in );
+            return 0;
+        }
+        public long decodeLong( ByteStoreAccess in ) throws IOException {
+            decodeObject( in );
+            return 0;
+        }
+        public double decodeDouble( ByteStoreAccess in ) throws IOException {
+            decodeObject( in );
+            return Double.NaN;
+        }
+        public boolean decodeBoolean( ByteStoreAccess in ) throws IOException {
+            decodeObject( in );
+            return false;
+        }
+    }
+
     /**
      * Codec which de/serialises fixed-length arrays.
      */
-    private static class FixedArrayCodec extends Codec {
+    private static class FixedArrayCodec extends ObjectCodec {
         final int nel_;
         final Codec1 codec1_;
         final int itemSize_;
@@ -347,7 +629,7 @@ abstract class Codec {
             return itemSize_;
         }
 
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             Object value = codec1_.getBuffer( nel_ );
             for ( int i = 0; i < nel_; i++ ) {
                 codec1_.decode1( value, i, in );
@@ -365,7 +647,7 @@ abstract class Codec {
      * elements is written as an integer value ahead of the data bytes
      * themselves.
      */
-    private static class VariableArrayCodec extends Codec {
+    private static class VariableArrayCodec extends ObjectCodec {
         final Codec1 codec1_;
 
         VariableArrayCodec( Codec1 codec1 ) {
@@ -381,7 +663,7 @@ abstract class Codec {
             return 4 + nel * codec1_.getItemSize1();
         }
 
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             int nel = in.readInt();
             if ( nel < 0 ) {
                 this.warnCorrupt();
@@ -404,7 +686,7 @@ abstract class Codec {
         }
     }
 
-    private static class FixedStringCodec extends Codec {
+    private static class FixedStringCodec extends ObjectCodec {
 
         final int nchar_;
         final char[] cbuf_;
@@ -429,7 +711,7 @@ abstract class Codec {
             return nchar_ * 2;
         }
 
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             int lastNonZero = -1;
             for ( int ic = 0; ic < nchar_; ic++ ) {
                 char c = in.readChar();
@@ -455,14 +737,14 @@ abstract class Codec {
                                         : ((String) value).toCharArray();
             return super.encode( cval, out );
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
-            char[] cval = (char[]) super.decode( in );
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
+            char[] cval = (char[]) super.decodeObject( in );
             return cval == null ? null
                                 : new String( cval );
         }
     }
 
-    private static class StringArrayCodec extends Codec {
+    private static class StringArrayCodec extends ObjectCodec {
         public int encode( Object value, DataOutput out ) throws IOException {
             int count = 0;
             String[] strings = value == null ? new String[ 0 ] 
@@ -482,7 +764,7 @@ abstract class Codec {
             }
             return count;
         }
-        public Object decode( ByteStoreAccess in ) throws IOException {
+        public Object decodeObject( ByteStoreAccess in ) throws IOException {
             int nstr = in.readInt();
             if ( nstr < 0 ) {
                 warnCorrupt();
@@ -514,7 +796,7 @@ abstract class Codec {
     }
 
     /**
-     * Abstract helper class defining an object which cn serialize
+     * Abstract helper class defining an object which can serialize
      * and deserialize elements of an array.
      */
     private static abstract class Codec1 {
