@@ -11,9 +11,8 @@ import java.io.IOException;
  * <ol>
  * <li>Zero or more calls of {@link #acceptCell}</li>
  * <li>A call of {@link #endCells}</li>
- * <li>Zero or more calls of {@link #readCell}</li>
- * <li>Optionally, a call to {@link #dispose}</li>
- * <ol>
+ * <li>Zero or more calls of {@link #createReader}</li>
+ * </ol>
  * Behaviour will be undefined if you violate this sequence.
  *
  * @author   Mark Taylor
@@ -30,22 +29,15 @@ interface ColumnStore {
 
     /**
      * Signals that no more calls to <code>acceptCell</code> will be made,
-     * and that calls to <code>readCell</code> may be made.
+     * and that calls to <code>createReader</code> may be made.
      */
     void endCells() throws IOException;
 
     /**
-     * Retrieves a datum from this store.
-     * The supplied index corresponds to the sequence in which the values
-     * were written using {@link #acceptCell}.
+     * Returns an object that can provide random access to the
+     * cells written to this store.
      *
-     * @param  lrow  index of datum to retrieve
-     * @return  the <code>lrow</code><sup>th</sup> written value
+     * @return   column cell reader
      */
-    Object readCell( long lrow ) throws IOException;
-
-    /**
-     * Releases any resources.  This object may not subsequently be used.
-     */
-    void dispose();
+    ColumnReader createReader();
 }
