@@ -46,6 +46,7 @@ import uk.ac.starlink.table.formats.IpacTableBuilder;
 import uk.ac.starlink.table.formats.IpacTableWriter;
 import uk.ac.starlink.table.formats.TstTableBuilder;
 import uk.ac.starlink.table.formats.TstTableWriter;
+import uk.ac.starlink.table.gui.TableSaveChooser;
 import uk.ac.starlink.util.DataSource;
 import uk.ac.starlink.util.FileDataSource;
 import uk.ac.starlink.util.IntList;
@@ -185,6 +186,45 @@ public class FormatsTest extends TableCase {
         assertTableEquals( table, new WrapperStarTable( table ) );
     }
 
+    public void testHandlerNames() throws TableFormatException {
+        StarTableOutput tout = new StarTableOutput();
+        String[] fnames = new String[] {
+            "fits-basic", "fits-plus", "fits-var", "fits-healpix",
+            "colfits-basic", "colfits-plus",
+            "votable", "ecsv", "feather", "text", "ascii", "csv",
+            "ipac", "tst", "html", "latex", "mirage",
+        };
+        for ( String fname : fnames ) {
+            assertNotNull( tout.getHandler( fname ) );
+            assertNotNull( tout.getHandler( fname + "()" ) );
+        }
+
+        StarTableFactory tfact = new StarTableFactory();
+        String[] bnames = new String[] {
+            "fits", "colfits-basic", "colfits-plus",
+            "votable", "cdf", "ecsv", "feather",
+        };
+        for ( String bname : bnames ) {
+            assertNotNull( tfact.getTableBuilder( bname ) );
+            assertNotNull( tfact.getTableBuilder( bname + "()" ) );
+        }
+    }
+
+    public void testLegacyOutputHandlers() throws TableFormatException {
+        StarTableOutput sto = new StarTableOutput();
+        for ( String hname :
+              StarTableOutput.createLegacyHandlerMap().keySet() ) {
+            assertNotNull( hname, sto.getHandler( hname ) );
+        }
+    }
+
+    public void testGuiOutputHandlers() throws TableFormatException {
+        StarTableOutput sto = new StarTableOutput();
+        for ( String hname : TableSaveChooser.getExtraWriterNames( false ) ) {
+            assertNotNull( hname, sto.getHandler( hname ) );
+        }
+    }
+
     public void testStorage() throws IOException {
         exerciseRowStore( new ListRowStore() );
         exerciseRowStore( new DiskRowStore() );
@@ -281,33 +321,22 @@ public class FormatsTest extends TableCase {
             }
         }
         String[] knownFormats = new String[] {
-            "fits",
             "fits-plus",
             "fits-basic",
             "fits-var",
             "fits-healpix",
             "colfits-plus",
             "colfits-basic",
-            "votable-tabledata",
-            "votable-binary-inline",
-            "votable-binary2-inline",
-            "votable-fits-href",
-            "votable-binary-href",
-            "votable-binary2-href",
-            "votable-fits-inline",
-            "ecsv-space",
-            "ecsv-comma",
+            "votable",
+            "ecsv",
             "feather",
             "text",
             "ascii",
             "csv",
-            "csv-noheader",
             "ipac",
             "tst",
             "html",
-            "html-element",
             "latex",
-            "latex-document",
             "mirage",
         };
         String[] gotFormats = new String[ handlers.size() ];
@@ -451,12 +480,10 @@ public class FormatsTest extends TableCase {
             ColumnInfo c2 = t2.getColumnInfo( ic );
             int[] shape1 = c1.getShape();
             int[] shape2 = c2.getShape();
-            String tform1 =
-                (String) c1.getAuxDatumValue( BintableStarTable.TFORM_INFO,
-                                              String.class );
-            String tform2 =
-                (String) c2.getAuxDatumValue( BintableStarTable.TFORM_INFO,
-                                              String.class );
+            String tform1 = c1.getAuxDatumValue( BintableStarTable.TFORM_INFO,
+                                                 String.class );
+            String tform2 = c2.getAuxDatumValue( BintableStarTable.TFORM_INFO,
+                                                 String.class );
             if ( shape1 != null && shape1[ shape1.length - 1 ] < 0 ) {
                 nvar++;
                 assertTrue( shape2[ shape2.length - 1 ] < 0 );
